<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/generation/GeneratorPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
define( "SORT_NAME", "name" );
define( "SORT_SIZE", 'size' );
define( "SORT_LAST_MODIFIED", 'lastmodified' );
define( "SORT_DIRECTORY", 'directory' );
    
class DirectoryGenerator extends GeneratorPipeElement implements PipeElementInterface
{
    
   
   
   private $gDepth = 1;
   
   private $gStartDir = 1;
   
   private $gDateFormat = ""; 
   
   
   private $gReverse = false;
   
   private $gInclude = "/.*/";
   
   private $gExclude = "";
   
   
   
    
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
        try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      $expandedString = StringResolver::expandString( $inVariableStack, $this->gSrc );
      $expandedString = StringResolver::expandPseudoProtocols( $expandedString );
      $this->gStartDir = $expandedString;
      
      if ( $this->gParameters->getParameter( 'dateFormat' ) != "" ) {
         $this->gDateFormat = $this->gParameters->getParameter( 'dateFormat' );
      }
      if ( $this->gParameters->getParameter( 'depth' ) != "" ) {
         $this->gDepth = $this->gParameters->getParameter( 'depth' );
      }
      if ( $this->gParameters->getParameter( 'reverse' ) != "" ) {
         $this->gReverse = $this->gParameters->getParameter( 'reverse' );
      }
      if ( $this->gParameters->getParameter( 'include' ) != "" ) {
         $this->gInclude = $this->gParameters->getParameter( 'include' );
      }
      if ( $this->gParameters->getParameter( 'exclude' ) != "" ) {
         $this->gExclude = $this->gParameters->getParameter( 'exclude' );
      }
      
      $documentDOM = new DomDocument();
      
      if ( file_exists( $this->gStartDir ) && is_dir( $this->gStartDir ) ) {
         $rootElement = $documentDOM->appendChild( $documentDOM->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:directory' ) );
         $rootElement->setAttribute( "name" , $this->gStartDir );
         $rootElement->setAttribute( "lastmodified" , $this->getLastModified($this->gStartDir) );
         $rootElement->setAttribute( "reverse" , $this->gReverse );
         $rootElement->setAttribute( "requested" , "true" );
         $rootElement->setAttribute( "size" , filesize( $this->gStartDir ) );
         $childElement = $this->scanDirectory( $documentDOM, $rootElement, $this->gStartDir, $this->gDepth );
         $rootElement->appendChild( $childElement );
      } else {
           throw new RunException( "File '{$this->gStartDir}' does not exist", PalooseException::SITEMAP_GENERATOR_RUN_ERROR );
      }
      return $documentDOM;
   }
   
   
   
   
   private function scanDirectory( $inDOMDocument, $inParentElement, $inDirectory, $inLevel )
   {
      
      $inLevel--;
      $element = NULL;
      
      $dirFileListing = scandir( $inDirectory, ( $this->gReverse === false ) ? 0 : 1 );
      foreach( $dirFileListing as $key => $value ) {
         $fullName = "$inDirectory/$value";
         if ( $value == '.'
               || $value == '..'
               || preg_match( $this->gInclude, $value ) == 0
               || preg_match( $this->gExclude, $value ) != 0 ) {
            
         } else if ( is_dir( $fullName ) ) {
            $element = $inParentElement->appendChild( $inDOMDocument->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:directory' ) );
            $element->setAttribute( "name" , $fullName );
            $element->setAttribute( "lastmodified" , $this->getLastModified( $fullName ) );
            $element->setAttribute( "size" , filesize( $fullName ) );
            if ( $inLevel > 0 ) {
               $scanString = $this->scanDirectory( $inDOMDocument, $element, $fullName, $inLevel );
            } else {
               $scanString = "";
            }
         } else {
            $element = $inParentElement->appendChild( $inDOMDocument->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:file' ) );
            $element->setAttribute( "name" , $value );
            $element->setAttribute( "lastmodified" , $this->getLastModified( $fullName ) );
            $element->setAttribute( "size" , filesize( $fullName ) );
         }
      }
      return $element;
   }
   
   
   
   private function getLastModified( $inDirectory )
   {
      return date( $this->gDateFormat, filemtime( $inDirectory ));
   }
   
   
   
   
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _DirectoryGenerator extends Generater {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "DirectoryGenerator";
   }
}
?>
