<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/matching/Matcher.php" );
class Matchers {
   private $gDefault;
   
   
   private $gMatchers;
   
   private $gDOM;
      
   
   
   
   public function __construct()
   {
      $this->gMatchers = array();
   }
   
   
   
   public function parse( $inNode )
   {
      
      $this->gDOM = new DOMDocument;
      $this->gDOM->appendChild( $this->gDOM->importNode( $inNode, 1 ) );
   
      
      $xpath = new domxpath( $this->gDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $component = $xpath->query( "//m:matcher" );
      
      foreach ( $component as $node ) { 
         $name = $node->getAttribute( "name" );
         $src = $node->getAttribute( "src" );
         $isCachable = Utilities::normalizeBoolean( $node->getAttribute( "cachable" ) );
         if ( strlen( $name ) == 0 or strlen( $src ) == 0 ) {
            throw new UserException(
               "Sitemap component matcher must have name and src defined",
               PalooseException::SITEMAP_MATCH_PARSE_ERROR,
               $this->gDOM );
         }
         
         
         if ( strstr( $src, Environment::getPathSeparator() ) == 1 ) {
            $packageFilename = $src;
         } else if ( !strstr( $src, ":" . Environment::getPathSeparator() ) ) {
            
            $packageFilename = PALOOSE_LIB_DIRECTORY . Environment::getPathSeparator() . $src;
         } else {
            
            $packageFilename = StringResolver::expandPseudoProtocols( $src );
         }
         
         $package = Utilities::stripFileName( $packageFilename );
         $path = Utilities::stripPath( StringResolver::expandPseudoProtocols( $packageFilename ) );
         
         
         if ( array_key_exists( $name, $this->gMatchers ) ) {
         } else {
         }   
         
         require_once( $path . $package . ".php" );
         $matcher = new Matcher( $name, $package, $node, $isCachable );
         
         try {
            $matcher->isValid();
            $this->gMatchers[ $name ] = $matcher;
         } catch( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $this->gDOM );
         }
      }
   }
   
   
   
   public function setDefault( $inDefault )
   {
      if ( !( array_key_exists( $inDefault, $this->gMatchers ) ) ) {
           throw new UserException(
              "Default matcher '". $inDefault . "' not found",
              PalooseException::SITEMAP_MATCH_PARSE_ERROR,
              $this->gDOM );
      }
      $this->gDefault = $inDefault;
   }
   
   
   
   public function getDefault()
   {
      return $this->gDefault;
   }
   
   
   
   public function getMatcherClassName( $inName )
   {
      if ( !( array_key_exists( $inName, $this->gMatchers ) ) ) {
           throw new UserException(  "Matcher '". $inName . "' not found", PalooseException::SITEMAP_MATCH_PARSE_ERROR, $this->gDOM );
      } else {
         $className = $this->gMatchers[ $inName ]->getPackageName();
         return $className;
      }
   }
   
   
   
   public function toString()
   {
      $mess = "  <matchers default='{$this->gDefault}'>\n";
      foreach ( $this->gMatchers as $node ) {
         $mess .= $node->toString();
      }
      $mess .= "  </matchers>\n";
      return $mess;
   }
}
?>
