<?php
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/RunException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/caching/CacheFile.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Aggregator.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Mount.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Redirect.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/resources/Call.php" );
class PipeElement
{
   
   const UNKNOWN = -1;
   const GENERATOR = 0;
   const AGGREGATOR = 1;
   const MOUNT = 2;
   const TRANSFORMER = 3;
   const SERIALIZER = 4;
   const READER = 5;
   const SELECTOR = 6;
   const CALL = 7;
   const ACTION = 8;
   const REDIRECT = 8;
   
   protected $gGenericType;
   
   protected $gType;
   
   protected $gSrc;
   
   protected $gParameters;
   
   protected $gInputDOM;
      
   protected $gOutputDOM;
      
   protected $gLabel;
   
    
   protected $gComponent;
    
   protected $gIsCachable = 0;
   
   protected $gCacheFile;
   
   protected $gParameterCacheFile;
       
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      $this->gParameters = new Parameter();
      
      $xpath = new domxpath( $inDOM );
      
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $this->gLabel = Utilities::getXPathListStringItem( 0, $xpath, "//@label" );
      $params = $xpath->query( "//m:parameter" );
      
      foreach ( $params as $node ) {
         $name = $node->getAttribute( "name" );
         $value = $node->getAttribute( "value" );
         if ( strlen( $name ) > 0 and strlen( $value ) > 0 ) {
            $this->gParameters->setParameter( $name, $value );
         } else {
            $dom = new DOMDocument;
            $dom->appendChild( $dom->importNode( $node, 1 ) );
            throw new UserException( "Parameters must have both 'name' and 'value' attributes",
                  PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR,
                  $dom );
         }
      }
      
      
      if ( $inComponent == null ) {
         $this->gIsCachable = 0;
      } else {
         $this->gIsCachable = $inComponent->gIsCachable;
      }
      
      
      $localCachable = $this->gParameters->getParameter( CacheFile::CACHABLE_ATTRIBUTE_NAME );
      if ( $localCachable != NULL ) {
         $this->gIsCachable = $localCachable;  
      } else {
         $this->gIsCachable = 0;
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $userReqParams = $this->gComponent->getUseRequestParametersFlag();
      
      if ( $userReqParams ) {
         $requestParameterModule = Environment::$modules[ 'request-param' ];
         $params = $requestParameterModule->getParams();
         foreach ( $params as $name => $value ) {
            $this->gParameters->setParameter( $name, $value );
         }
      }
      
      if ( sizeof( $this->gParameters->getParameterList() ) > 0  ) {
         $parameterList = $this->gParameters->getParameterList();
         foreach ( $parameterList as $name => $value ) {
            $expandedValue = StringResolver::expandString( $inVariableStack, $value );
            $expandedValue = StringResolver::expandPseudoProtocols( $expandedValue );
            $this->gParameters->setParameter( $name, $expandedValue );
         }
      }
      return $inDOM;
   }
   
   
   
   public function getType()
   {
      return $this->gType;
   } 
   
   
   public function getSrc()
   {
      return $this->gSrc;
   } 
   
   
   public function getGenericType()
   {
      return $this->gGenericType;
   } 
   
   
   public function getLabel()
   {
      return $this->gLabel;
   } 
   
   
   
 
   public static function createGenerator( $inNode )
   {
      $generators = Environment::$sitemapStack->root()->getGenerators();
      $matchDom = new DOMDocument;
      $matchDom->appendChild( $matchDom->importNode( $inNode, 1 ) );
      
      $src = $inNode->getAttribute( "src" );
      if ( strlen( $src ) == 0 ) {
         $dom = new DOMDocument;
         $dom->appendChild( $dom->importNode( $inNode, 1 ) );
         throw new UserException( "Must have a src attribute present", PalooseException::SITEMAP_GENERATOR_PARSE_ERROR, $dom );
      }
      $type = $inNode->getAttribute( "type" );
      
      if ( strlen( $type ) == 0 ) {
         $type = $generators->getDefault();
      }
      $gIsCachable = Utilities::normalizeBoolean( $inNode->getAttribute( "cache" ) );
      
      if ( strlen( $gIsCachable ) == 0 ) {
         $gIsCachable = true;
      }
      try {
         $generator = $generators->getGenerator( $type );
         $generatorPackageName = $generator->getPackageName();
         return new $generatorPackageName( $matchDom, $type, $src, $generator );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
         throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
    
   public static function createAggregator( $inNode )
   {
      $matchDom = new DOMDocument;
      $matchDom->appendChild( $matchDom->importNode( $inNode, 1 ) );
      try {
         return new Aggregator( $matchDom );
      } catch ( UserException $e ) {
         throw new UserException( "Error in aggregator with '" . $this->gPattern . "' pattern : \n" . $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   
    
   public static function createTransformer( DOMNode $inNode )
   {
      $transformers = Environment::$sitemapStack->root()->getTransformers();
      $dom = new DOMDocument;
      $dom->appendChild( $dom->importNode( $inNode, 1 ) );
      
      $src = $inNode->getAttribute( "src" );
      $type = $inNode->getAttribute( "type" );
      
      if ( strlen( $type ) == 0 ) {
         $type = $transformers->getDefault();
      }
      
      try {
         $transformer = $transformers->getTransformer( $type );
         $transformerPackageName = $transformer->getPackageName();
         return new $transformerPackageName( $dom, $type, $src, $transformer );
      } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
           throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   } 
   
   
   
   public static function createSerializer( DOMNode $inNode )
   {
      $serializers = Environment::$sitemapStack->root()->getSerializers();
      $dom = new DOMDocument;
      $dom->appendChild( $dom->importNode( $inNode, 1 ) );
      
      
      $attributes = $inNode->attributes;
      $type = $inNode->getAttribute( "type" );
      
      if ( strlen( $type ) == 0 ) {
         $type = $serializers->getDefault();
      }
   
      
      try {
         $serializer = $serializers->getSerializer( $type );
         $serializerPackageName = $serializer->getPackageName();
         return new $serializerPackageName( $dom, $type, "", $serializer );
      } catch ( InternalException $e ) {
           throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
   public static function createReader( $inNode )
   {
      $readers = Environment::$sitemapStack->root()->getReaders();
      $dom = new DOMDocument;
      $dom->appendChild( $dom->importNode( $inNode, 1 ) );
      
      $src = $inNode->getAttribute( "src" );
      if ( strlen( $src ) == 0 ) {
           throw new UserException( "Must have a src attribute present",
           PalooseException::SITEMAP_READER_PARSE_ERROR,
           $dom );
        }
      $type = $inNode->getAttribute( "type" );
      
      if ( strlen( $type ) == 0 ) {
         $type = $readers->getDefault();
      }
      
      try {
         $reader = $readers->getReader( $type );
         $readerPackageName = $reader->getPackageName();
         return new $readerPackageName( $dom, $type, $src, $reader );
      } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
           throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
    
   public static function createSelector( $inNode )
   {
      
      $selectors = Environment::$sitemapStack->root()->getSelectors();
      $dom = new DOMDocument;
      $dom->appendChild( $dom->importNode( $inNode, 1 ) );
      $type = $inNode->getAttribute( "type" );
      
      if ( strlen( $type ) == 0 ) {
         $type = $selectors->getDefault();
      }
      try {
         $selector = $selectors->getSelector( $type );
         $selectorPackageName = $selector->getPackageName();
         return new $selectorPackageName( $dom, $type, "", $selector );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
         throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
    
   public static function createAction( $inNode )
   {
      
      $actions = Environment::$sitemapStack->root()->getActions();
      $type = $inNode->getAttribute( "type" );
      
      if ( strlen( $type ) == 0 ) {
         $type = $actions->getDefault();
      }
      $dom = new DOMDocument;
      $dom->appendChild( $dom->importNode( $inNode, 1 ) );
      try {
         $action = $actions->getAction( $type );
         $actionPackageName = $action->getPackageName();
         return new $actionPackageName( $dom, $type, "", $action );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $dom );
      } catch ( InternalException $e ) {
         throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
    
   public static function createRedirect( $inNode )
   {
      
      try {
         $dom = new DOMDocument;
         $dom->appendChild( $dom->importNode( $inNode, 1 ) );
         return new Redirect( $dom, "", $inNode->getAttribute( "src" ) );
      } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
           throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
    
   public static function createMount( $inNode )
   {
      
      try {
         $dom = new DOMDocument;
         $dom->appendChild( $dom->importNode( $inNode, 1 ) );
         return new Mount( $dom, "", $inNode->getAttribute( "src" ) );
      } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
           throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
    
   public static function createCall( $inNode )
   {
      
      try {
         $dom = new DOMDocument;
         $dom->appendChild( $dom->importNode( $inNode, 1 ) );
         return new Call( $dom, "", "", NULL );
      } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( InternalException $e ) {
           throw new InternalException( $e->getMessage(), $e->getCode() );
      }
   }
  
   
   
  
   public function toString( $inGenericType )
   {
      switch ( $inGenericType ) {
         case PipeElement::REDIRECT : $tagName = "redirect"; break;
         case PipeElement::GENERATOR : $tagName = "generate"; break;
         case PipeElement::AGGREGATOR : $tagName = "aggregate"; break;
         case PipeElement::MOUNT : $tagName = "mount"; break;
         case PipeElement::TRANSFORMER : $tagName = "transform"; break;
         case PipeElement::SERIALIZER : $tagName = "serialize"; break;
         case PipeElement::READER : $tagName = "read"; break;
         case PipeElement::SELECTOR : $tagName = "select"; break;
         case PipeElement::CALL : $tagName = "call"; break;
      }
   
      if ( sizeof( $this->gParameters->getParameterList() ) > 0  ) {
         $parameterList = $this->gParameters->getParameterList();
         $mess = "     <$tagName type='{$this->gType}' package='" . get_class( $this ) . "'";
         if ( strlen( $this->gLabel ) ) {
            $mess .= " label='{$this->gLabel}'";
         }
         $mess .= ">\n";
         foreach ( $parameterList as $name => $value ) {
            $mess .= "       <parameter name='$name' value='$value'/>\n";
         }
         reset( $this->gParameters );
         $mess .= "     </$tagName>\n";
      } else {
         $mess = "     <$tagName type='{$this->gType}' package='" . get_class( $this ) . "'";
         if ( strlen( $this->gLabel ) ) {
            $mess .= " label='{$this->gLabel}'";
         }
         $mess .= "/>\n";
      }
      return $mess;
   }
   
}
?>
