<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/selection/Selector.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
class Selectors {
   private $gDefault;
   
   private $Selectors;
   
   private $gDOM;
      
   
   
   
   public function __construct()
   {
      $this->gSelectors = array();
   }
   
   
   
   public function parse( DOMNode $inNode )
   {
      
      $this->gDOM = new DOMDocument;
      $this->gDOM->appendChild( $this->gDOM->importNode( $inNode, 1 ) );
   
      
      $xpath = new domxpath( $this->gDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $component = $xpath->query( "//m:selector" );
      
      foreach ( $component as $node ) { 
         $name = $node->getAttribute( "name" );
         $src = $node->getAttribute( "src" );
         $isCachable = Utilities::normalizeBoolean( $node->getAttribute( "cachable" ) );
         if ( !strlen( $name ) or !strlen( $src ) ) {
            throw new UserException(
               "Component selector must have name and src attributes defined",
               PalooseException::SITEMAP_SELECTOR_PARSE_ERROR,
               $this->gDOM );
         }
         
         
         if ( strstr( $src, Environment::getPathSeparator() ) == 1 ) {
            $packageFilename = $src;
         } else if ( !strstr( $src, ":" . Environment::getPathSeparator() ) ) {
            
            $packageFilename = PALOOSE_LIB_DIRECTORY . Environment::getPathSeparator() . $src;
         } else {
            
            $packageFilename = StringResolver::expandPseudoProtocols( $src );
         }
         $package = Utilities::stripFileName( $packageFilename );
         
         $path = Utilities::stripPath( StringResolver::expandPseudoProtocols( $packageFilename ) );
         
         
         if ( array_key_exists( $name, $this->gSelectors ) ) {
         } else {
         }
         
         require_once( $path . $package . ".php" );
         
         $selectorName = "_" . $package;
         $selector = new $selectorName( $name, $path.$package, $node, $isCachable );
         
         try {
            $selector->isValid();
            $this->gSelectors[ $name ] = $selector;
         } catch( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $this->gDOM );
         }
      }
   }
   
   
   
   public function setDefault( $inDefault )
   {
      if ( !( array_key_exists( $inDefault, $this->gSelectors ) ) ) {
           throw new UserException(
              "Default selector '". $inDefault . "' not found",
              PalooseException::SITEMAP_SELECTOR_PARSE_ERROR,
              $this->gDOM );
      }
      $this->gDefault = $inDefault;
   }
   
   
   
   public function getDefault()
   {
      return $this->gDefault;
   }
   
   
   
   public function getSelector( $inName )
   {
      if ( !( array_key_exists( $inName, $this->gSelectors ) ) ) {
           throw new InternalException(
              "Selector '". $inName . "' not found", 
              PalooseException::SITEMAP_SELECTOR_PARSE_ERROR,
              $this->gDOM );
      } else {
         return $this->gSelectors[ $inName ];
      }
   }
   
   
   
   public function getSelectorClassName( $inType )
   {
      if ( !( array_key_exists( $inType, $this->gSelectors ) ) ) {
           throw new InternalException( 
              "Selector '". $inType . "' not found",
              PalooseException::SITEMAP_SELECTOR_PARSE_ERROR,
              $this->gDOM );
      } else {
         $className = $this->gSelectors[ $inType ]->getPackageName();
         return $className;
      }
   }
   
   
   
   public function toString()
   {
      if ( $this->gSelectors == NULL ) return "";
      $mess = "  <selectors default='{$this->gDefault}'>\n";
      foreach ( $this->gSelectors as $node ) {
         $mess .= $node->toString();
      }
      $mess .= "  </selectors>\n";
      return $mess;
   }
}
?>
