<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/serialization/Serializer.php" );
class Serializers {
   private $gDefault;
   
   
   private $gSerializers;
   
   private $gDOM;
      
   
   
   public function __construct()
   {
      $this->gSerializers = NULL;
   }
   
   
   
   public function parse( DOMNode $inNode )
   {
      
      $this->gDOM = new DOMDocument;
      $this->gDOM->appendChild( $this->gDOM->importNode( $inNode, 1 ) );
   
      
      $xPath = new domxpath( $this->gDOM );
      $xPath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $component = $xPath->query( "//m:serializer" );
      
      foreach ( $component as $node ) { 
         $name = $node->getAttribute( "name" );
         $src = $node->getAttribute( "src" );
         $isCachable = Utilities::normalizeBoolean( $node->getAttribute( "cachable" ) );
         if ( !strlen( $name ) or !strlen( $src ) ) {
            throw new UserException(
               "Sitemap component serializer must have name and src attributes defined",
               PalooseException::SITEMAP_SERIALIZER_PARSE_ERROR, 
               $this->gDOM );
         }
         
         
         
         if ( strstr( $src, Environment::getPathSeparator() ) == 1 ) {
            $packageFilename = $src;
         } else if ( !strstr( $src, ":" . Environment::getPathSeparator() ) ) {
            
            $packageFilename = PALOOSE_LIB_DIRECTORY . Environment::getPathSeparator() . $src;
         } else {
            
            $packageFilename = StringResolver::expandPseudoProtocols( $src );
         }
         
         $package = Utilities::stripFileName( $packageFilename );
         $path = Utilities::stripPath( StringResolver::expandPseudoProtocols( $packageFilename ) );
         
         
         if ( $this->gSerializers and array_key_exists( $name, $this->gSerializers ) ) {
         } else {
         }
         
         require_once( $path . $package. ".php" );
         
         $serializerName = "_" . $package;
         $serializer = new $serializerName( $name, $path.$package, $node, $isCachable );
         
         try {
            $serializer->isValid();
            $this->gSerializers[ $name ] = $serializer;
         } catch( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $this->gDOM );
         }
      }
   }
   
   
   
   public function setDefault( $inDefault )
   {
      if ( !array_key_exists( $inDefault, $this->gSerializers ) ) {
           throw new UserException(
              "Default serializer '". $inDefault . "' not found",
              PalooseException::SITEMAP_SERIALIZER_PARSE_ERROR,
              $this->gDOM );
      }
      $this->gDefault = $inDefault;
      return true;
   }
   
   public function getDefault()
   {
      return $this->gDefault;
   }
  
   
   
   public function getSerializer( $inType )
   {
      if ( !( array_key_exists( $inType, $this->gSerializers ) ) ) {
           throw new UserException(
              "Serializer '". $inType . "' not found",
              PalooseException::SITEMAP_SERIALIZER_PARSE_ERROR,
              $this->gDOM );
      } else {
         return $this->gSerializers[ $inType ];
      }
   }
   
   
   
   public function getSerializerClassName( $inType )
   {
      if ( !( array_key_exists( $inType, $this->gSerializers ) ) ) {
           throw new UserException(
              "Serializer class '". $inType . "' not found",
              PalooseException::SITEMAP_SERIALIZER_PARSE_ERROR,
              $this->gDOM );
      } else {
         return $this->gSerializers[ $inType ]->getPackageName();
      }
   }
   
   
   public function getCount()
   {
      return count( $this->gSerializers );
   }
  
   
   public function toString()
   {
      $mess = "  <serializers default='{$this->gDefault}'>\n";
      foreach ( $this->gSerializers as $node ) {
         $mess .= $node->toString();
      }
      $mess .= "  </serializers>\n";
      return $mess;
   }
}
?>
