<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/authentication/EncryptMD5.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
 
class PasswordTransformer extends TransformerPipeElement implements PipeElementInterface
{
      
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
       try {
          parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
       } catch ( UserException $e ) {
          throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
       }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $rootNode = $inDOM->documentElement;
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      $this->processNode( $inVariableStack, $rootNode, $newElement );
      return $this->gTranslatedDOM;
   }
   
   
   
   private function processNode( VariableStack $inVariableStack, DOMNode $inParentNode, DOMNode &$inNewParentNode )
   {
      
      $autheticateTagFound = false;
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         $encodedAttribs = array();
         if ( $inParentNode->hasAttributes() and $inParentNode->localName == "authentication" ) {
            $autheticateTagFound = true;
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = $attribute->name;
               $attribValue = $attribute->value;
               $encodedAttribs[ $attribName ] = $attribValue;
            }
         } else if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         }
      }
      
      
      
      if ( $autheticateTagFound === true ) {
         if ( $encodedAttribs[ "password" ] != "" ) {
            $encodedPassword = EncryptMD5::encrypt( $encodedAttribs[ "username" ], $encodedAttribs[ "password" ] );
         } else {
            $encodedPassword = "";
         }
         $encodedAttribs[ "password" ] = $encodedPassword;
      }
      
      foreach ( $encodedAttribs as $name => $value ) {
         $inNewParentNode->setAttribute( $name, $value );
      }
 
      
      
      $children = $inParentNode->childNodes;
      foreach ( $children as $child ) {
         if ( $child->nodeType == XML_ELEMENT_NODE ) {
            
            $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
            
            $this->processNode( $inVariableStack, $child, $newElementNode );
            $inNewParentNode->appendChild( $newElementNode );
         } else if ( $child->nodeType == XML_TEXT_NODE ) {
            
            $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
            $inNewParentNode->appendChild( $newTextNode );
         }
      }
   }
   
   
   
  
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _PasswordTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "PasswordTransformer";
   }
}
?>
