<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
 
class SQLTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
   
   public $gHost;
   
    
   public $gUser;
   
    
   public $gPassword;
   
       
   private $gShowNumberOfRows;
   
   public $gDatabaseType;
   
   const  DBX_GENERIC = 0;
   const  DBX_MYSQL = 1;
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
        try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
       
        
        $parameter = $this->gParameters->getParameter( 'host' );
        if ( strlen( $parameter ) > 0 ) {
         $this->gHost = $parameter;
      } else {
         $this->gHost = StringResolver::expandPseudoProtocols( $inComponent->gHost );
      }
        $parameter = $this->gParameters->getParameter( 'user' );
        if ( strlen( $parameter ) > 0 ) {
         $this->gUser = $parameter;
      } else {
         $this->gUser = StringResolver::expandPseudoProtocols( $inComponent->gUser );
      }
       $parameter = $this->gParameters->getParameter( 'password' );
        if ( strlen( $parameter ) > 0 ) {
         $this->gPassword = $parameter;
      } else {
         $this->gPassword = StringResolver::expandPseudoProtocols( $inComponent->gPassword );
      }
       $parameter = $this->gParameters->getParameter( 'show-nr-of-rows' );
        if ( strlen( $parameter ) > 0 ) {
         $this->gShowNumberOfRows = Utilities::normalizeBoolean( $parameter );
      } else {
         $this->gShowNumberOfRows = false;
      }
      
      $this->gDatabaseType = $inComponent->gDatabaseType;
    }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      
      
      $documentDOM = $inDOM;
      $rootNode = $documentDOM->documentElement;
      $this->gTranslatedDOM = new DOMDocument( '1.0', 'UTF-8' );
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      try {
         $this->processNode( $inVariableStack, $rootNode, $newElement, Environment::$configuration[ 'sqlNamespace' ] );
         $this->gTranslatedDOM->formatOutput = true;
         return $this->gTranslatedDOM;
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
   
   
   
   private function processNode( VariableStack $inVariableStack, DOMNode $inParentNode, DOMNode &$inNewParentNode, $inSQLNamespace )
   {
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         }
      }
      
      
      $children = $inParentNode->childNodes;
      
      try {
         foreach ( $children as $child ) {
            if ( $child->nodeType == XML_ELEMENT_NODE ) {
            }
            if ( $child->localName == "execute-query" && $child->namespaceURI == $inSQLNamespace ) {
               
               
               $this->processNode( $inVariableStack, $child, $inNewParentNode, $inSQLNamespace );
            } else if ( $child->localName == "query" && $child->namespaceURI == $inSQLNamespace ) {
               $query = $child->nodeValue;
               
               $query = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $query ) );
               
              
               
               
               
               try {
                  $queryName = $child->getAttribute( 'name' );
                  $databaseName = $child->getAttribute( 'database' );
                  if ( strlen( $databaseName ) == 0 ) {
                     throw new RunException( "No SQL database specified for query: ", PalooseException::SQL_ERROR_TYPE, NULL );
                  }
                  $replacementDOM = $this->queryMySQLDatabase( $this->gHost, $this->gUser, $this->gPassword, $databaseName, $queryName, $query, $inSQLNamespace );
                  
                  
                  
                  
                  $replacementNode = $replacementDOM->documentElement;
                  $newNode = $this->gTranslatedDOM->importNode( $replacementNode, true );
                  
                  $inNewParentNode->appendChild( $newNode );
               } catch ( RunException $e ) {
                  
                  $replacementDOM = $this->formatErrorFragment( $query, $e->getMessage(), $inSQLNamespace );
                  $replacementRoot = $replacementDOM->documentElement;
                  $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
                  $inNewParentNode->appendChild( $newElementNode );
               }
            } else { 
               
               if ( $child->nodeType == XML_ELEMENT_NODE ) {
                 
                 $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
                 
                 $this->processNode( $inVariableStack, $child, $newElementNode, $inSQLNamespace );
                 $inNewParentNode->appendChild( $newElementNode );
               } else if ( $child->nodeType == XML_TEXT_NODE ) {
                  
                  $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
                  $inNewParentNode->appendChild( $newTextNode );
               }
            }
         }
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
  
   
   
   
  
   private function queryMySQLDatabase( $inHost, $inUser, $inPassword, $inDatabaseName, $inQueryName, $inQuery, $inSQLNamespace )
   {
      $link = mysql_connect( $inHost, $inUser, $inPassword );
      if ( !$link ) {
         throw new RunException( "SQL connect error: " . mysql_error(), PalooseException::SQL_ERROR_TYPE, NULL );
      }
       $db_selected = mysql_select_db( $inDatabaseName, $link );
      if ( !$db_selected ) {
         mysql_close( $link );
         throw new RunException( "SQL cannot use database [" . $inDatabaseName . "] :" . mysql_error(), PalooseException::SQL_ERROR_TYPE, NULL );
      }
      $result = mysql_query( $inQuery );
      if ( !$result ) {
         mysql_close( $link );
         throw new RunException( " SQL query error: " . mysql_error(), PalooseException::SQL_ERROR_TYPE, NULL );
      }
      $dom = new DOMDocument();
      
      if ( $result === true ) {
         
         $root = $dom->createElementNS( $inSQLNamespace, 'result', "true" );
         $rootNode = $dom->appendChild( $root );
         $rootNode->setAttribute( "name", $inQueryName );
      } else {
         
         $root = $dom->createElementNS( $inSQLNamespace, 'row-set' );
         $rootNode = $dom->appendChild( $root );
         $numOfRows = 0;
         while ( $row = mysql_fetch_assoc( $result ) ) {
            $numOfRows++;
            $rowElement = $dom->createElementNS( $inSQLNamespace, 'row' );
            $rowNode = $root->appendChild( $rowElement );
            foreach ( $row as $name => $value  ) {
               
               $encoding = mb_detect_encoding( $value, "auto" );
               if ( $encoding == "UTF-8" ) {
                  if ( Utilities::is_utf8( $value ) ) {
                     $normalisedValue = $value;
                  } else {
                     $normalisedValue = utf8_encode( $value );
                  }
               } else if ( $encoding == "ASCII" ) {
                  $normalisedValue = $value;
               }
               $rowElement->appendChild( $dom->createElementNS( $inSQLNamespace, $name, $normalisedValue ) );     
            }
         }
         $rootNode->setAttribute( "nrofrows", $numOfRows );
         $rootNode->setAttribute( "name", $inQueryName );
      }
      mysql_close( $link );
      return $dom;
   }
   
   
   
  
   private function formatErrorFragment( $inQuery, $inMessage, $inSQLNamespace )
   {
      
      $dom = new DOMDocument();
      $root = $dom->createElementNS( $inSQLNamespace, 'sql-error' );
      $dom->appendChild( $root );
      $root->appendChild( $dom->createElementNS( $inSQLNamespace, 'host', $this->gHost ) );
      $root->appendChild( $dom->createElementNS( $inSQLNamespace, 'user', $this->gUser  ) );
      $root->appendChild( $dom->createElementNS( $inSQLNamespace, 'password', $this->gPassword  ) );
      $root->appendChild( $dom->createElementNS( $inSQLNamespace, 'message', $inMessage . " => query: " . $inQuery ) );
      return $dom;
   }
   
   
   
  
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _SQLTransformer extends Transformer {
    
   
    
   public $gHost;
   
    
   public $gUser;
   
    
   public $gPassword;
   
   
   public $gDatabaseType;
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "SQLTransformer";
      
      $this->gHost = "";
      if ( strlen( $this->gParameters[ 'host' ] ) > 0 ) $this->gHost = $this->gParameters[ 'host' ];
      $this->gUser = "";
      if ( strlen( $this->gParameters[ 'user' ] ) > 0 ) $this->gUser = $this->gParameters[ 'user' ];
      $this->gPassword = "";
      if ( strlen( $this->gParameters[ 'password' ] ) > 0 ) $this->gPassword = $this->gParameters[ 'password' ];
      $this->gDatabaseType = SQLTransformer::DBX_GENERIC;
      if ( strlen( $this->gParameters[ 'type' ] ) > 0 ) {
         $type = $this->gParameters[ 'type' ];
         if ( $type == 'mysql' ) {
            $this->gDatabaseType = SQLTransformer::DBX_MYSQL;
         }
      }
      
   }
}
?>
