<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
 
class SourceWritingTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
   const WRITE_SOURCE_ACTION_NEW = 1;
   const WRITE_SOURCE_ACTION_OVERWRITTEN = 2;
   const WRITE_SOURCE_ACTION_NONE = 3;
   
   const WRITE_SOURCE_BEHAVIOUR_WRITE = 1;
   const WRITE_SOURCE_BEHAVIOUR_INSERT = 2;
   const WRITE_SOURCE_BEHAVIOUR_DELETE = 3;
   const WRITE_SOURCE_EXECUTION_SUCCESS = 1;
   const WRITE_SOURCE_EXECUTION_FAILURE = 2;
   const WRITE_SOURCE_SERIALIZER_XML = 1;
    
   private $gCreateFlag = false;
   
   
   private $gOverwriteFlag = true;
   
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
       try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      
      $documentDOM = $inDOM;
      $rootNode = $documentDOM->documentElement;
      
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      try {
         $this->processNode( $inVariableStack, null, $rootNode, $newElement );
         return $this->gTranslatedDOM;
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
      
   }
   
   
   
   private function processNode( VariableStack $inVariableStack, $inCatalogueDOM, DOMNode $inParentNode, DOMNode &$inNewParentNode )
   {
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
               
            }
         }
      }
      
      
      $children = $inParentNode->childNodes;
      
      try {
         foreach ( $children as $child ) {
            if ( $child->nodeName == "source:write" ) {
               $replacementDOM = $this->processWrite( $child );
               
               $replacementRoot = $replacementDOM->documentElement;
               
               $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
               $inNewParentNode->appendChild( $newElementNode );
            } else if ( $child->nodeName == "source:insert" ) {
               $replacementDOM = $this->processInsert( $child );
               
               $replacementRoot = $replacementDOM->documentElement;
               
               $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
               $inNewParentNode->appendChild( $newElementNode );
            } else if ( $child->nodeName == "source:delete" ) {
               $replacementDOM = $this->processDelete( $child );
               
               $replacementRoot = $replacementDOM->documentElement;
               
               $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
               $inNewParentNode->appendChild( $newElementNode );
            } else {               
               if ( $child->nodeType == XML_ELEMENT_NODE ) {
                 
                 $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
                 
                 $this->processNode( $inVariableStack, $inCatalogueDOM, $child, $newElementNode );
                 $inNewParentNode->appendChild( $newElementNode );
               } else if ( $child->nodeType == XML_TEXT_NODE ) {
                  
                  $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
                  $inNewParentNode->appendChild( $newTextNode );
               }
            }
         }
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
   
   
   
   
  
   private function processWrite( DOMNode $inSourceNode )
   {
      
      $dom = new DOMDocument();
      $dom->appendChild( $dom->importNode( $inSourceNode, 1 ) );
      
      
      $xpath = new DOMXpath( $dom );
      $xpath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      $filename = Utilities::getXPathListStringItem( 0, $xpath, "//source:source" );
      $filename = StringResolver::expandPseudoProtocols( $filename );
      $childXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:path" );
      $this->gCreateFlag = Utilities::normalizeBoolean( Utilities::getXPathListStringItem( 0, $xpath, "//@create" ) );
      
      if ( !file_exists( $filename ) and !$this->gCreateFlag ) {
         throw new RunException( "No file to receive source write fragment (try create attrib?)",
            PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
      }
      
      
      $fragmentDOM = new DOMDocument();
      $fragmentDOM->appendChild( $fragmentDOM->importNode( Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" ), 1 ) );
      
      $sourceDOM = new DOMDocument();
      try {
         if ( strlen( $childXpath ) > 0 ) {
            $pathXML = $this->xpathToXML( $childXpath );
            $sourceDOM->loadXML( $pathXML );
            $sourceXpath = new DOMXpath( $sourceDOM );
            
            $parentNode = Utilities::getXPathListNode( 0, $sourceXpath, $childXpath );
            if ( $parentNode == NULL ) {
            }
            
            $sourceDOM->importNode( $fragmentDOM->documentElement, true );
            $parentNode->appendChild( $sourceDOM->importNode( $fragmentDOM->documentElement, true ) );
            
            
            if ( fopen( $filename, "wb" ) === false ) {
               throw new RunException( "Cannot write source write fragment (directory permissions?)",
                  PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
            }
            
         } else {
            
            $nodeList = $xpath->query( "//source:fragment/*" );
            if ( $nodeList->length == 1 ) {
               $sourceDOM->loadXML( $fragmentDOM->saveXML() );
            } else {
               throw new RunException( "Fragment must have one root element if no path declared",
                  PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
            }
         }
      
         
         if ( !$sourceDOM->save( $filename ) ) {
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                  self::WRITE_SOURCE_BEHAVIOUR_WRITE,
                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                  self::WRITE_SOURCE_SERIALIZER_XML,
                  $filename,
                  "Could not save document to file '$filename': " . $php_errormsg
               );
         } else {
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                  self::WRITE_SOURCE_BEHAVIOUR_WRITE,
                  self::WRITE_SOURCE_EXECUTION_SUCCESS,
                  self::WRITE_SOURCE_SERIALIZER_XML,
                  $filename,
                  "File written ok"
               );
          }
      
      } catch ( Exception $e ) {
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
               self::WRITE_SOURCE_BEHAVIOUR_WRITE,
               self::WRITE_SOURCE_EXECUTION_FAILURE,
               self::WRITE_SOURCE_SERIALIZER_XML,
               $filename,
               "Problem processing source document in write-source: " . $e->getMessage()
            );
      }
      return $resultDOM;
   }
   
   
   
  
   private function processInsert( DOMNode $inSourceNode )
   {
      
      $dom = new DOMDocument();
      $dom->appendChild( $dom->importNode( $inSourceNode, 1 ) );
      
      
      $xpath = new DOMXpath( $dom );
      $xpath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      $filename = Utilities::getXPathListStringItem( 0, $xpath, "//source:source" );
      $filename = StringResolver::expandPseudoProtocols( $filename );
      $childXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:path" );
      $replaceXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:replace" );
      
      $this->gCreateFlag = Utilities::normalizeBoolean( Utilities::getXPathListStringItem( 0, $xpath, "//@create" ) );
      $this->gOverwriteFlag = Utilities::normalizeBoolean( Utilities::getXPathListStringItem( 0, $xpath, "//@overwrite" ) );
      $replaceFlag = ( strlen( $replaceXpath ) > 0 );
      
      
      
  
      
      $replaceXpath = $childXpath . "/" . $replaceXpath;
      
      
      if ( strlen( $filename ) == 0 ) {
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
               self::WRITE_SOURCE_BEHAVIOUR_INSERT,
               self::WRITE_SOURCE_EXECUTION_FAILURE,
               self::WRITE_SOURCE_SERIALIZER_XML,
               $filename,
               "Must have <source:source> defined in <source:insert>:" . $e->getMessage() );
         return $resultDOM;
      }
      if ( strlen( $childXpath ) == 0 ) {
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
               self::WRITE_SOURCE_BEHAVIOUR_INSERT,
               self::WRITE_SOURCE_EXECUTION_FAILURE,
               self::WRITE_SOURCE_SERIALIZER_XML,
               $filename,
               "Must have <source:path> defined in <source:insert>:" . $e->getMessage() );
         return $resultDOM;
      }
      
       
      if ( !file_exists( $filename ) and !$this->gCreateFlag ) {
         throw new RunException( "No file to receive source write fragment (try @create attrib in source:insert?)",
            PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
      }
      
      $fragmentDOM = new DOMDocument();
      
      $writeFileFlag = false;
      
      $sourceDOM = new DOMDocument();
      if ( file_exists( $filename ) ) {
         if ( !$sourceDOM->load( $filename ) ) {
            throw new RunException( "Could not load document from file '$filename'", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
         } else {
            
            $sourceXpath = new DOMXpath( $sourceDOM );
            $replaceNodeExists = ( $replaceFlag and $sourceXpath->query( $replaceXpath )->length > 0 );
            
            
            try {
               if ( !$replaceFlag ) {
                  
                  if ( Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" ) == NULL ) {
                     throw new RunException( "No fragment specified for source insert", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
                  }
                  $fragmentDOM->appendChild( $fragmentDOM->importNode( Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" ), 1 ) );
                  
                  $parentNode = Utilities::getXPathListNode( 0, $sourceXpath, $childXpath );
                  
                  $sourceDOM->importNode( $fragmentDOM->documentElement, true );
                  $parentNode->appendChild( $sourceDOM->importNode( $fragmentDOM->documentElement, true ) );
                  $writeFileFlag = true;
               } else if ( $replaceFlag and $replaceNodeExists and $this->gOverwriteFlag ) {
                  
                  
                  $nodeToReplace = Utilities::getXPathListNode( 0, $sourceXpath, "//$replaceXpath" );
                  if ( $nodeToReplace == NULL ) {
                     throw new Exception( "Cannot find node to delete: $replaceXpath" );
                  }
                  
                  $parentNode = Utilities::getXPathListNode( 0, $sourceXpath, "//$replaceXpath/parent::*" );
                  if ( $parentNode == NULL ) {
                     throw new Exception( "Cannot find parent of node to replace: $replaceXpath" );
                  }
                  $fragmentNode = Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" );
                  if ( $fragmentNode == NULL ) {
                     
                     $parentNode->removeChild( $nodeToReplace );
                     
                  } else {
                     $replaceNode = $sourceDOM->importNode( $fragmentNode, 1 );
                     $parentNode->replaceChild( $replaceNode, $nodeToReplace );
                     
                  }
                  $writeFileFlag = true;
               } else if ( $replaceFlag and !$this->gOverwriteFlag ) {
                  
                  $writeFileFlag = false;
               } else if ( $replaceFlag and !$replaceNodeExists ) {
                  
                  
                   if ( Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" ) == NULL ) {
                     throw new RunException( "No fragment specified for source insert", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
                  }
                  $fragmentDOM->appendChild( $fragmentDOM->importNode( Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" ), 1 ) );
                  
                  $parentNode = Utilities::getXPathListNode( 0, $sourceXpath, $childXpath );
                  
                  $sourceDOM->importNode( $fragmentDOM->documentElement, true );
                  $parentNode->appendChild( $sourceDOM->importNode( $fragmentDOM->documentElement, true ) );
                  $writeFileFlag = true;
               }
               
               
               
               if ( $writeFileFlag ) {
                  
                  if ( !$sourceDOM->save( $filename ) ) {
                     $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                           self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                           self::WRITE_SOURCE_EXECUTION_FAILURE,
                           self::WRITE_SOURCE_SERIALIZER_XML,
                           $filename,
                           "Could not save document to file '$filename': " . $php_errormsg );
                  } else {
                     $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                           self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                           self::WRITE_SOURCE_EXECUTION_SUCCESS,
                           self::WRITE_SOURCE_SERIALIZER_XML,
                           $filename,
                           "File written ok" );
                   }
               } else {
                  $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                        self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                        self::WRITE_SOURCE_EXECUTION_FAILURE,
                        self::WRITE_SOURCE_SERIALIZER_XML,
                        $filename,
                        "Did not write to '$filename': " );
               }
            } catch ( Exception $e ) {
               $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                     self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                     self::WRITE_SOURCE_EXECUTION_FAILURE,
                     self::WRITE_SOURCE_SERIALIZER_XML,
                     $filename,
                     "Problem processing source document in write-source:" . $e->getMessage() );
            }
         }
      } else {
           throw new RunException( "File '$filename' does not exist", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
      }
      return $resultDOM;
   }
   
   
   
  
   private function processDelete( DOMNode $inSourceNode )
   {
      
      $dom = new DOMDocument();
      $dom->appendChild( $dom->importNode( $inSourceNode, 1 ) );
      
      
      $xpath = new DOMXpath( $dom );
      $xpath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      
      $filename = StringResolver::expandPseudoProtocols( Utilities::getXPathListStringItem( 0, $xpath, "//source:source" ) );
      $childXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:path" );
      $replaceXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:replace" );
      try {
         if ( !file_exists( $filename ) ) {
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
               self::WRITE_SOURCE_BEHAVIOUR_DELETE,
               self::WRITE_SOURCE_EXECUTION_FAILURE,
               self::WRITE_SOURCE_SERIALIZER_XML,
               $filename,
               "File does not exist" );
         } else {
            
            if ( !unlink( $filename ) ) {
               $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                  self::WRITE_SOURCE_BEHAVIOUR_DELETE,
                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                  self::WRITE_SOURCE_SERIALIZER_XML,
                  $filename,
                  "Could not delete file '$filename': " . $php_errormsg );
            } else {
               $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
                  self::WRITE_SOURCE_BEHAVIOUR_DELETE,
                  self::WRITE_SOURCE_EXECUTION_SUCCESS,
                  self::WRITE_SOURCE_SERIALIZER_XML,
                  $filename,
                  "File delete ok" );
            }
         }
      } catch ( Exception $e ) {
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW, 
            self::WRITE_SOURCE_BEHAVIOUR_DELETE,
            self::WRITE_SOURCE_EXECUTION_FAILURE,
            self::WRITE_SOURCE_SERIALIZER_XML,
            $filename,
            "Problem processing source document in write-source: " . $e->getMessage() );
      }
      return $resultDOM;
   }
   
   
   
  
   private function formatResultFragment( $inAction, $inBehaviour, $inExecution, $inSerializer, $inSource, $inMessage )
   {
      $ns = Environment::$configuration[ 'sourceNamespace' ];
      
      $dom = new DOMDocument();
      $root = $dom->createElementNS( $ns, 'sourceResult' );
      $dom->appendChild( $root );
      switch ( $inAction ) {
         case self::WRITE_SOURCE_ACTION_NEW : $action = 'new'; break;
         case self::WRITE_SOURCE_ACTION_OVERWRITTEN : $action = 'overwritten'; break;
         case self::WRITE_SOURCE_ACTION_NONE : $action = 'none'; break;
         default: $action = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'action', $action ) );
      
      switch ( $inBehaviour ) {
         case self::WRITE_SOURCE_BEHAVIOUR_WRITE : $behaviour = 'write'; break;
         case self::WRITE_SOURCE_BEHAVIOUR_INSERT : $behaviour = 'insert'; break;
         default: $behaviour = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'behaviour', $behaviour ) );
      switch ( $inExecution ) {
         case self::WRITE_SOURCE_EXECUTION_SUCCESS : $execution = 'success'; break;
         case self::WRITE_SOURCE_EXECUTION_FAILURE : $execution = 'failure'; break;
         default: $execution = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'execution', $execution ) );
      switch ( $inSerializer ) {
         case self::WRITE_SOURCE_SERIALIZER_XML : $serializer = 'xml'; break;
         default: $serializer = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'serializer', $serializer ) );
      $root->appendChild( $dom->createElementNS( $ns, 'source', $inSource ) );
      $root->appendChild( $dom->createElementNS( $ns, 'message', $inMessage ) );
      return $dom;
   }
   
   
   
  
   private function xpathToXML( $inXpathExpr )
   {
      $firstTag = "";
      $remainder = "";
      preg_match( "%/([^/]+)(.*)%", $inXpathExpr, $matches );
      if ( isset( $matches[ 1 ] ) ) { $firstTag = $matches[ 1 ]; }
      if ( isset( $matches[ 2 ] ) ) { $remainder = $matches[ 2 ]; }
      if ( strlen( $remainder ) > 0 ) {
         return "<$firstTag>" . $this->xpathToXML( $remainder ) . "</$firstTag>";
      } else {
         return "<$firstTag/>";
      }
   }
   
   
   
   
  
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _SourceWritingTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "SourceWritingTransformer";
   }
}
?>
