<?php
class View {
       
       
   private $gName;
       
   private $gFromLabel;
    
   private $gFromPosition;
    
   private $gPipe;
   
   
   
   public function __construct( $inName, $inFromLabel, $inFromPosition, DOMNode $inNode )
   {
       $this->gName = $inName;
       $this->gFromLabel = $inFromLabel;
       $this->gFromPosition = $inFromPosition;
      if ( !( strlen( $inName ) > 0 and ( strlen( $inFromLabel ) > 0 or strlen( $inFromPosition ) > 0 ) ) ) {
           $viewDom = new DOMDocument;
           $viewDom->appendChild( $viewDom->importNode( $inNode, 1 ) );
           throw new UserException( "Sitemap view declaration must have name and either from-label or from-position defined",
                                     PalooseException::SITEMAP_VIEW_PARSE_ERROR, 
                                     $viewDom );
      }
   }
   
   
   
    
   public function parse( DOMNode $inNode )
   {
      
      $viewDom = new DOMDocument;
      $viewDom->appendChild( $viewDom->importNode( $inNode, 1 ) );
      
      $xpath = new domxpath( $viewDom );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      $component = $xpath->query( "//m:view/*" );
      $this->gPipe = new Pipe();
      foreach ( $component as $pipeNode ) {
         $name = $pipeNode->localName;
         
         switch ( $name ) {
            case "redirect-to" :
            case "mount" :
            case "aggregate" :
            case "generate" :
            case "read" :
                 throw new UserException( "View pipe cannot have generation/redirect components", PalooseException::SITEMAP_VIEW_PARSE_ERROR, $viewDom );
            case "transform" :
               try {
                  $transformer = PipeElement::createTransformer( $pipeNode );
                  
                  $this->gPipe->addComponent( $transformer );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "serialize" :
               try {
                  $serializer = PipeElement::createSerializer( $pipeNode );
                  $this->gPipe->addComponent( $serializer );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            
            
            case "call" :
               try {
                  $call = PipeElement::createCall( $pipeNode );
                  $this->gPipe->addComponent( $call );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
         }
      }
   }
   
   
   
   public function getName()
   {
      return $this->gName;
   }
   
   
   
   public function getFromLabel()
   {
      return $this->gFromLabel;
   }
   
   
   
   public function getFromPosition()
   {
      return $this->gFromPosition;
   }
   
   
   
   public function getPipe()
   {
      return $this->gPipe;
   }
   
   
   
    
   public function toString()
   {
      $mess = "   <view name='{$this->gName}'";
      if ( strlen( $this->gFromLabel ) > 0 )
         $mess .= " from-label='{$this->gFromLabel}'";
      if ( strlen( $this->gFromPosition ) > 0 )
         $mess .= " from-position='{$this->gFromPosition}'";
      $mess .= ">\n";
      $mess .= $this->gPipe->toString();
      $mess .= "  </view>\n";
      return $mess;
   }
}
?>
